@@ -7,16 +7,18 @@ Python version by Stuart Colville http://muffinresearch.co.uk
 License: http://www.opensource.org/licenses/mit-license.php
 """
 
-from __future__ import unicode_literals
-
 import argparse
+import logging
+logger = logging.getLogger(__name__)
+import os
+import re
 import string
 import sys
 
 import regex
 
 __all__ = ['titlecase']
-__version__ = '1.0.0'
+__version__ = '1.1.0'
 
 SMALL = r'a|an|and|as|at|but|by|en|for|if|in|of|on|or|the|to|v\.?|via|vs\.?'
 PUNCT = r"""!"“#$%&'‘()*+,\-–‒—―./:;?@[\\\]_`{|}~"""
@@ -36,14 +38,9 @@ MAC_MC = regex.compile(r"^([Mm]c|MC)(\w.+)")
 class Immutable(object):
     pass
 
-
-text_type = unicode if sys.version_info < (3,) else str
-
-
-class ImmutableString(text_type, Immutable):
+class ImmutableString(str, Immutable):
     pass
 
-
 class ImmutableBytes(bytes, Immutable):
     pass
 
@@ -65,7 +62,28 @@ def set_small_word_list(small=SMALL):
     SUBPHRASE = regex.compile(r'([:.;?!][ ])(%s)' % small)
 
 
-def titlecase(text, callback=None, small_first_last=True):
+def create_wordlist_filter(path_to_config=None):
+    """
+    This function checks for a default list of abbreviations which need to 
+    remain as they are (e.g. uppercase only or mixed case).
+    The file is retrieved from ~/.titlecase.txt (platform independent)
+    """
+    if path_to_config is None:
+        path_to_config = os.path.join(os.path.expanduser('~'), ".titlecase.txt")
+    if not os.path.isfile(str(path_to_config)):
+        logger.debug('No config file found at ' + str(path_to_config))
+        return lambda word, **kwargs : None
+    with open(str(path_to_config)) as f:
+        logger.debug('Config file used from ' + str(path_to_config))
+        abbreviations = [abbr.strip() for abbr in f.read().splitlines() if abbr]
+        abbreviations_capitalized = [abbr.upper() for abbr in abbreviations]
+        for abbr in abbreviations:
+            logger.debug("This acronym will be kept as written here: " + abbr)
+        return lambda word, **kwargs : (abbreviations[abbreviations_capitalized.index(word.upper())]
+                                       if word.upper() in abbreviations_capitalized else None)
+
+
+def titlecase(text, callback=None, small_first_last=True, wordlist_file=None):
     """
     :param text: Titlecases input text
     :param callback: Callback function that returns the titlecase version of a specific word
@@ -81,6 +99,7 @@ def titlecase(text, callback=None, small_first_last=True):
     the New York Times Manual of Style, plus 'vs' and 'v'.
 
     """
+    wordlist_filter = create_wordlist_filter(wordlist_file)
 
     lines = regex.split('[\r\n]+', text)
     processed = []
@@ -97,6 +116,12 @@ def titlecase(text, callback=None, small_first_last=True):
                     tc_line.append(_mark_immutable(new_word))
                     continue
 
+            # If the user has a custom wordlist, defer to that
+            new_word = wordlist_filter(word, all_caps=all_caps)
+            if new_word:
+                tc_line.append(_mark_immutable(new_word))
+                continue
+
             if all_caps:
                 if UC_INITIALS.match(word):
                     tc_line.append(word)
@@ -177,7 +202,9 @@ def titlecase(text, callback=None, small_first_last=True):
 
         processed.append(result)
 
-    return "\n".join(processed)
+    result = "\n".join(processed)
+    logger.debug(result)
+    return result
 
 
 def cmd():
@@ -195,6 +222,8 @@ def cmd():
             help='File to read from to titlecase')
     parser.add_argument('-o', '--output-file',
             help='File to write titlecased output to')
+    parser.add_argument('-w', '--wordlist',
+            help='Wordlist for acronyms')
 
     args = parser.parse_args()
 
@@ -221,4 +250,4 @@ def cmd():
             in_string = ifile.read()
 
     with ofile:
-        ofile.write(titlecase(in_string))
+        ofile.write(titlecase(in_string, wordlist_file=args.wordlist))
